// based on Wireshark commit 193150fdb728cd70a0ceb8e48af259575a0b2a7b

#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <errno.h>

#include <linux/un.h>

#define MAX_MCS_INDEX 76

/* For IEEE80211_RADIOTAP_MCS */
#define IEEE80211_RADIOTAP_MCS_HAVE_BW		0x01
#define IEEE80211_RADIOTAP_MCS_HAVE_MCS		0x02
#define IEEE80211_RADIOTAP_MCS_HAVE_GI		0x04
#define IEEE80211_RADIOTAP_MCS_HAVE_FMT		0x08
#define IEEE80211_RADIOTAP_MCS_HAVE_FEC		0x10
#define IEEE80211_RADIOTAP_MCS_HAVE_STBC	0x20
#define IEEE80211_RADIOTAP_MCS_HAVE_NESS	0x40
#define IEEE80211_RADIOTAP_MCS_NESS_BIT1	0x80

#define IEEE80211_RADIOTAP_MCS_BW_MASK		0x03
#define		IEEE80211_RADIOTAP_MCS_BW_20	0
#define		IEEE80211_RADIOTAP_MCS_BW_40	1
#define		IEEE80211_RADIOTAP_MCS_BW_20L	2
#define		IEEE80211_RADIOTAP_MCS_BW_20U	3
#define IEEE80211_RADIOTAP_MCS_SGI		0x04
#define IEEE80211_RADIOTAP_MCS_FMT_GF		0x08
#define IEEE80211_RADIOTAP_MCS_FEC_LDPC		0x10
#define IEEE80211_RADIOTAP_MCS_STBC_MASK	0x60
#define IEEE80211_RADIOTAP_MCS_STBC_SHIFT	5
#define		IEEE80211_RADIOTAP_MCS_STBC_1	1
#define		IEEE80211_RADIOTAP_MCS_STBC_2	2
#define		IEEE80211_RADIOTAP_MCS_STBC_3	3
#define IEEE80211_RADIOTAP_MCS_NESS_BIT0	0x80

/* For IEEE80211_RADIOTAP_VHT */
#define IEEE80211_RADIOTAP_VHT_HAVE_STBC	0x0001
#define IEEE80211_RADIOTAP_VHT_HAVE_TXOP_PS	0x0002
#define IEEE80211_RADIOTAP_VHT_HAVE_GI		0x0004
#define IEEE80211_RADIOTAP_VHT_HAVE_SGI_NSYM_DA	0x0008
#define IEEE80211_RADIOTAP_VHT_HAVE_LDPC_EXTRA	0x0010
#define IEEE80211_RADIOTAP_VHT_HAVE_BF		0x0020
#define IEEE80211_RADIOTAP_VHT_HAVE_BW		0x0040
#define IEEE80211_RADIOTAP_VHT_HAVE_GID		0x0080
#define IEEE80211_RADIOTAP_VHT_HAVE_PAID	0x0100
#define IEEE80211_RADIOTAP_VHT_STBC		0x01
#define IEEE80211_RADIOTAP_VHT_TXOP_PS		0x02
#define IEEE80211_RADIOTAP_VHT_SGI		0x04
#define IEEE80211_RADIOTAP_VHT_SGI_NSYM_DA	0x08
#define IEEE80211_RADIOTAP_VHT_LDPC_EXTRA	0x10
#define IEEE80211_RADIOTAP_VHT_BF		0x20
#define IEEE80211_RADIOTAP_VHT_NSS		0x0f
#define IEEE80211_RADIOTAP_VHT_MCS		0xf0
#define IEEE80211_RADIOTAP_VHT_CODING_LDPC	0x01

#define IEEE80211_RADIOTAP_VHT_BW_MASK		0x1f
#define IEEE80211_RADIOTAP_VHT_BW_20		IEEE80211_RADIOTAP_MCS_BW_20
#define IEEE80211_RADIOTAP_VHT_BW_40		IEEE80211_RADIOTAP_MCS_BW_40
#define IEEE80211_RADIOTAP_VHT_BW_20L		IEEE80211_RADIOTAP_MCS_BW_20L
#define IEEE80211_RADIOTAP_VHT_BW_20U		IEEE80211_RADIOTAP_MCS_BW_20U
#define IEEE80211_RADIOTAP_VHT_BW_80		4
#define IEEE80211_RADIOTAP_VHT_BW_40L		5
#define IEEE80211_RADIOTAP_VHT_BW_40U		6
#define IEEE80211_RADIOTAP_VHT_BW_20LL		7
#define IEEE80211_RADIOTAP_VHT_BW_20LU		8
#define IEEE80211_RADIOTAP_VHT_BW_20UL		9
#define IEEE80211_RADIOTAP_VHT_BW_20UU		10
#define IEEE80211_RADIOTAP_VHT_BW_160		11
#define IEEE80211_RADIOTAP_VHT_BW_80L		12
#define IEEE80211_RADIOTAP_VHT_BW_80U		13
#define IEEE80211_RADIOTAP_VHT_BW_40LL		14
#define IEEE80211_RADIOTAP_VHT_BW_40LU		15
#define IEEE80211_RADIOTAP_VHT_BW_40UL		16
#define IEEE80211_RADIOTAP_VHT_BW_40UU		17
#define IEEE80211_RADIOTAP_VHT_BW_20LLL		18
#define IEEE80211_RADIOTAP_VHT_BW_20LLU		19
#define IEEE80211_RADIOTAP_VHT_BW_20LUL		20
#define IEEE80211_RADIOTAP_VHT_BW_20LUU		21
#define IEEE80211_RADIOTAP_VHT_BW_20ULL		22
#define IEEE80211_RADIOTAP_VHT_BW_20ULU		23
#define IEEE80211_RADIOTAP_VHT_BW_20UUL		24
#define IEEE80211_RADIOTAP_VHT_BW_20UUU		25

/*
 * Lookup for the MCS index (0-76)
 * returning the number of data bits per symbol
 * assumes 52 subcarriers (20MHz)
 * symbols are 4us for long guard interval, 3.6us for short guard interval
 * Note: MCS 32 is special - only valid for 40Mhz channel.
 */
const uint16_t ieee80211_ht_Dbps[MAX_MCS_INDEX+1] = {
    /* MCS  0 - 1 stream */
    26, 52, 78, 104, 156, 208, 234, 260,

    /* MCS  8 - 2 stream */
    52, 104, 156, 208, 312, 416, 468, 520,

    /* MCS 16 - 3 stream */
    78, 156, 234, 312, 468, 624, 702, 780,

    /* MCS 24 - 4 stream */
    104, 208, 312, 416, 624, 832, 936, 1040,

    /* MCS 32 - 1 stream */
    12, /* only valid for 40Mhz - 11a/g DUP mode */

    /* MCS 33 - 2 stream */
    156, 208, 260, 234, 312, 390,

    /* MCS 39 - 3 stream */
    208, 260, 260, 312, 364, 364, 416, 312, 390, 390, 468, 546, 546, 624,

    /* MCS 53 - 4 stream */
    260, 312, 364, 312, 364, 416, 468, 416, 468, 520, 520, 572,
    390, 468, 546, 468, 546, 624, 702, 624, 702, 780, 780, 858
};

/*
 * Calculates data rate corresponding to a given 802.11n MCS index,
 * bandwidth, and guard interval.
 */
static float ieee80211_htrate(int mcs_index, bool bandwidth, bool short_gi)
{
    return (float)(ieee80211_ht_Dbps[mcs_index] * (bandwidth ? 108 : 52) / 52.0 / (short_gi ? 3.6 : 4.0));
}

// https://www.radiotap.org/fields/MCS.html
static uint32_t calculate_htrate(uint8_t mcs_known, uint8_t mcs_flags, uint8_t mcs_index)
{
    if (!(mcs_known & IEEE80211_RADIOTAP_MCS_HAVE_MCS))
        return 0;

    bool bandwidth;
    if ((mcs_known & IEEE80211_RADIOTAP_MCS_HAVE_BW))
        bandwidth = ((mcs_flags & IEEE80211_RADIOTAP_MCS_BW_MASK) == IEEE80211_RADIOTAP_MCS_BW_40) ? true : false;
    else
        bandwidth = false; // if not found, estimate lower rate

    bool short_gi;
    if (mcs_known & IEEE80211_RADIOTAP_MCS_HAVE_GI)
        short_gi = (mcs_flags & IEEE80211_RADIOTAP_MCS_SGI) ? true : false;
    else
        short_gi = false; // if not found, estimate lower rate

    if (mcs_index > MAX_MCS_INDEX)
        return 0;

    return ieee80211_htrate(mcs_index, bandwidth, short_gi) * 10.0;
}

#define MAX_MCS_VHT_INDEX	9
#define MAX_VHT_NSS             8

/*
 * Maps a VHT bandwidth index to ieee80211_vhtinfo.rates index.
 */
static const int ieee80211_vht_bw2rate_index[] = {
		/*  20Mhz total */	0,
		/*  40Mhz total */	1, 0, 0,
		/*  80Mhz total */	2, 1, 1, 0, 0, 0, 0,
		/* 160Mhz total */	3, 2, 2, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0
};

struct mcs_vht_valid {
	bool valid[4][MAX_VHT_NSS]; /* indexed by bandwidth and NSS-1 */
};

#if 0
static const struct mcs_vht_valid ieee80211_vhtvalid[MAX_MCS_VHT_INDEX+1] = {
		/* MCS  0  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  1  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  2  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  3  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  4  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  5  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  6  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  false, true,  true,  true,  false, true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  7  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  8  */
		{
			{	/* 20 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 160 Mhz */ { true,  true,  true,  true,  true,  true,  true,  true },
			}
		},
		/* MCS  9  */
		{
			{	/* 20 Mhz */  { false, false, true,  false, false, true,  false, false },
				/* 40 Mhz */  { true,  true,  true,  true,  true,  true,  true,  true },
				/* 80 Mhz */  { true,  true,  true,  true,  true,  false, true,  true },
				/* 160 Mhz */ { true,  true,  false, true,  true,  true,  true,  true },
			}
		}
};
#endif

struct mcs_vht_info {
	const char *modulation;
	const char *coding_rate;
	float       rates[4][2]; /* indexed by bandwidth and GI length */
};

static const struct mcs_vht_info ieee80211_vhtinfo[MAX_MCS_VHT_INDEX+1] = {
		/* MCS  0  */
		{	"BPSK",		"1/2",
				{		/* 20 Mhz */  {    6.5f,		/* SGI */    7.2f, },
						/* 40 Mhz */  {   13.5f,		/* SGI */   15.0f, },
						/* 80 Mhz */  {   29.3f,		/* SGI */   32.5f, },
						/* 160 Mhz */ {   58.5f,		/* SGI */   65.0f, }
				}
		},
		/* MCS  1  */
		{	"QPSK",		"1/2",
				{		/* 20 Mhz */  {   13.0f,		/* SGI */   14.4f, },
						/* 40 Mhz */  {   27.0f,		/* SGI */   30.0f, },
						/* 80 Mhz */  {   58.5f,		/* SGI */   65.0f, },
						/* 160 Mhz */ {  117.0f,		/* SGI */  130.0f, }
				}
		},
		/* MCS  2  */
		{	"QPSK",		"3/4",
				{		/* 20 Mhz */  {   19.5f,		/* SGI */   21.7f, },
						/* 40 Mhz */  {   40.5f,		/* SGI */   45.0f, },
						/* 80 Mhz */  {   87.8f,		/* SGI */   97.5f, },
						/* 160 Mhz */ {  175.5f,		/* SGI */  195.0f, }
				}
		},
		/* MCS  3  */
		{	"16-QAM",	"1/2",
				{		/* 20 Mhz */  {   26.0f,		/* SGI */   28.9f, },
						/* 40 Mhz */  {   54.0f,		/* SGI */   60.0f, },
						/* 80 Mhz */  {  117.0f,		/* SGI */  130.0f, },
						/* 160 Mhz */ {  234.0f,		/* SGI */  260.0f, }
				}
		},
		/* MCS  4  */
		{	"16-QAM",	"3/4",
				{		/* 20 Mhz */  {   39.0f,		/* SGI */   43.3f, },
						/* 40 Mhz */  {   81.0f,		/* SGI */   90.0f, },
						/* 80 Mhz */  {  175.5f,		/* SGI */  195.0f, },
						/* 160 Mhz */ {  351.0f,		/* SGI */  390.0f, }
				}
		},
		/* MCS  5  */
		{	"64-QAM",	"2/3",
				{		/* 20 Mhz */  {   52.0f,		/* SGI */   57.8f, },
						/* 40 Mhz */  {  108.0f,		/* SGI */  120.0f, },
						/* 80 Mhz */  {  234.0f,		/* SGI */  260.0f, },
						/* 160 Mhz */ {  468.0f,		/* SGI */  520.0f, }
				}
		},
		/* MCS  6  */
		{	"64-QAM",	"3/4",
				{		/* 20 Mhz */  {   58.5f,		/* SGI */   65.0f, },
						/* 40 Mhz */  {  121.5f,		/* SGI */  135.0f, },
						/* 80 Mhz */  {  263.3f,		/* SGI */  292.5f, },
						/* 160 Mhz */ {  526.5f,		/* SGI */  585.0f, }
				}
		},
		/* MCS  7  */
		{	"64-QAM",	"5/6",
				{		/* 20 Mhz */  {   65.0f,		/* SGI */   72.2f, },
						/* 40 Mhz */  {  135.0f,		/* SGI */  150.0f, },
						/* 80 Mhz */  {  292.5f,		/* SGI */  325.0f, },
						/* 160 Mhz */ {  585.0f,		/* SGI */  650.0f, }
				}
		},
		/* MCS  8  */
		{	"256-QAM",	"3/4",
				{		/* 20 Mhz */  {   78.0f,		/* SGI */   86.7f, },
						/* 40 Mhz */  {  162.0f,		/* SGI */  180.0f, },
						/* 80 Mhz */  {  351.0f,		/* SGI */  390.0f, },
						/* 160 Mhz */ {  702.0f,		/* SGI */  780.0f, }
				}
		},
		/* MCS  9  */
		{	"256-QAM",	"5/6",
				{		/* 20 Mhz */  {   86.7f,		/* SGI */   96.3f, },
						/* 40 Mhz */  {  180.0f,		/* SGI */  200.0f, },
						/* 80 Mhz */  {  390.0f,		/* SGI */  433.3f, },
						/* 160 Mhz */ {  780.0f,		/* SGI */  866.7f, }
				}
		}
};

// https://www.radiotap.org/fields/VHT.html
static int32_t calculate_vhtrate(uint16_t known, uint8_t vht_flags, uint8_t bw, uint8_t *user)
{
    int gi_length;
    if (known & IEEE80211_RADIOTAP_VHT_HAVE_GI) {
        gi_length = (vht_flags & IEEE80211_RADIOTAP_VHT_SGI) ? 1 : 0;
    } else {
        gi_length = 0; // if not found, estimate lower rate
    }

    int bandwidth = 0; // if not found, estimate lower rate
    if (known & IEEE80211_RADIOTAP_VHT_HAVE_BW) {
        if (bw < sizeof(ieee80211_vht_bw2rate_index)/sizeof(ieee80211_vht_bw2rate_index[0]))
            bandwidth = ieee80211_vht_bw2rate_index[bw];
    }

    int32_t vht_rate = 0;
    bool found = false;
    for (int i = 0; i < 4; i++) {
        uint32_t nss = (user[i] & IEEE80211_RADIOTAP_VHT_NSS);
        if (!nss || nss > MAX_VHT_NSS)
            continue;

        uint32_t mcs = (user[i] & IEEE80211_RADIOTAP_VHT_MCS) >> 4;
        if (mcs > MAX_MCS_VHT_INDEX)
            continue;

#if 0
        if (!ieee80211_vhtvalid[mcs].valid[bandwidth][nss - 1]) {
            printf("invalid mcs=%d,bandwidth=%d,nss=%d\n", mcs, bandwidth, nss);
        }
#endif

        //printf("i=%d,mcs=%d,bandwidth=%d,gi_length=%d,nss=%d", i, mcs, bandwidth, gi_length, nss);
        int32_t rate = round(ieee80211_vhtinfo[mcs].rates[bandwidth][gi_length] * nss * 10.0);
        if (found) {
            // if multi rate found, use lower rate
            if (vht_rate > rate)
                vht_rate = rate;
        } else {
            found = true;
            vht_rate = rate;
        }
    }
    if (!found)
        return -1;

    return vht_rate;
}

#if TEST_ENABLED
static void __test__calculate_htrate(uint8_t mcs_known, uint8_t mcs_flags, uint8_t mcs_index, uint32_t expected)
{
    uint32_t res = calculate_htrate(mcs_known, mcs_flags, mcs_index);
    if (res != expected)
        printf("calculate_htrate(%d,%d,%d)=%d != %d\n", mcs_known, mcs_flags, mcs_index, res, expected);
}
static void __test__calculate_vhtrate(uint16_t known, uint8_t vht_flags, uint8_t bw, uint8_t *user, int32_t expected)
{
    int32_t vht_rate = calculate_vhtrate(known, vht_flags, bw, user);
    if (vht_rate != expected)
        printf("calculate_vhtrate()=%d != %d\n", vht_rate, expected);
}
#endif /* TEST_ENABLED */

static int init_server(char *socketPath)
{
	struct sockaddr_un server_address;
	int s;

	// make socket
	s = socket(PF_UNIX, SOCK_STREAM, 0);
	if (s < 0) {
		fprintf(stderr, "socket() failed(%d): %s\n", errno, strerror(errno));
		return -1;
	}

	// bind
	unlink(socketPath);
	memset(&server_address, 0, sizeof(server_address));
	server_address.sun_family = PF_UNIX;
	strncpy(server_address.sun_path, socketPath, sizeof(server_address.sun_path));
	server_address.sun_path[sizeof(server_address.sun_path) - 1] = '\0';
	if (bind(s, (struct sockaddr *)&server_address, sizeof(struct sockaddr_un)) < 0) {
		fprintf(stderr, "bind() failed(%d): %s\n", errno, strerror(errno));
		close(s);
		return -1;
	}

	// listen
	if (listen(s, 5) < 0) {
		fprintf(stderr, "listen() failed(%d): %s\n\n", errno, strerror(errno));
		close(s);
		return -1;
	}
	return s;
}

static int accept_socket(int socket)
{
	struct sockaddr_un client_address;
	socklen_t clientAddrLen = 0;
	int s;

	memset(&client_address, 0, sizeof(client_address));
	s = accept(socket, (struct sockaddr *)&client_address,
		&clientAddrLen);
	if (s < 0) {
		fprintf(stderr, "accept() failed(%d): %s", errno, strerror(errno));
		return -1;
	}
	//printf("accepted client name : [%s]\n" , client_address.sun_path);
	return s;
}

static int read_size(int s, uint8_t *buf, size_t size)
{
	size_t recved = 0;
	ssize_t ret;

	while (recved < size) {
		ret = read(s, &buf[recved], size - recved);
		if (ret < 0) {
			fprintf(stderr, "read() failed(%zd)\n", ret);
			return -1;
		}
		if (ret == 0) {
			//printf("socket closed\n");
			return 0;
		}
		recved += ret;
	}
	return size;
}

int main(void)
{
#if TEST_ENABLED
    printf("ratetest self test started\n");
    /* calculate_htrate */
    __test__calculate_htrate(0, 0, 0, 0);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 0, 65);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS | IEEE80211_RADIOTAP_MCS_HAVE_BW, 0, 0, 65);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS | IEEE80211_RADIOTAP_MCS_HAVE_BW, IEEE80211_RADIOTAP_MCS_BW_40, 0, 135);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS | IEEE80211_RADIOTAP_MCS_HAVE_GI, 0, 0, 65);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS | IEEE80211_RADIOTAP_MCS_HAVE_GI, IEEE80211_RADIOTAP_MCS_SGI, 0, 72);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 13, 1040);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 14, 1170);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 15, 1300);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 76, 2145);
    __test__calculate_htrate(IEEE80211_RADIOTAP_MCS_HAVE_MCS, 0, 77, 0);

    /* calculate_vhtrate */
    uint8_t user[4];
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, (uint8_t *)"\x00\x00\x00\x00", -1);
    memset(user, 0, sizeof(user));
    user[0] = 0x01;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, 65);
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_GI, 0x00, 0x00, user, 65);
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_GI, IEEE80211_RADIOTAP_VHT_SGI, 0x00, user, 72);
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_BW, 0x00, 0x01, user, 135);
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_BW, 0x00, 25, user, 65);
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_BW, 0x00, 26, user, 65);
    memset(user, 0, sizeof(user));
    user[0] = 0x11;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, 130);
    user[0] = 0x19;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, -1);
    memset(user, 0, sizeof(user));
    user[0] = 0xa1;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, -1);
    memset(user, 0, sizeof(user));
    user[3] = 0x01;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, 65);
    user[3] = 0x11;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, 130);
    memset(user, 0, sizeof(user));
    user[0] = 0x11;
    user[3] = 0x21;
    __test__calculate_vhtrate(0x0000, 0x00, 0x00, user, 130);

    memset(user, 0, sizeof(user));
    user[3] = 0x91;
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_GI | IEEE80211_RADIOTAP_VHT_HAVE_BW, IEEE80211_RADIOTAP_VHT_SGI, 11, user, 8667);
    user[3] = 0x98;
    __test__calculate_vhtrate(IEEE80211_RADIOTAP_VHT_HAVE_GI | IEEE80211_RADIOTAP_VHT_HAVE_BW, IEEE80211_RADIOTAP_VHT_SGI, 11, user, 69336);

    __test__calculate_vhtrate(0x01c5, 0x10, 0x00, (uint8_t *)"\x00\x72\x00\x00", 1300);
    printf("ratetest self test ended\n");
#endif /* TEST_ENABLED */

#define BUFFER_SIZE 32
	uint8_t buf[BUFFER_SIZE];
	int server_socket, client_socket;

#define PACKET_PARSER_SOCKET_DIR "/var/run/visual_wireless"
	if (mkdir(PACKET_PARSER_SOCKET_DIR, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH) < 0) {
		if (errno != EEXIST) {
			fprintf(stderr, "mkdir() failed(%d): %s\n", errno, strerror(errno));
			return -1;
		}
	}

#define PACKET_PARSER_SOCKET_PATH PACKET_PARSER_SOCKET_DIR "/socket"
	server_socket = init_server(PACKET_PARSER_SOCKET_PATH);
	if (server_socket < 0)
		return -1;

	while (true) {
		client_socket = accept_socket(server_socket);
		if (client_socket < 0)
			goto out;

		//printf("client accepted\n");

		while (true) {
			int ret = read_size(client_socket, buf, 1);
			//printf("read ret=%d,buf[0]=0x%02x\n", ret, buf[0]);
			if (ret < 1)
				break;

			int type = buf[0];
			int32_t rate;
			if (type == 0) {
				ret = read_size(client_socket, buf, 3);
				if (ret < 1)
					break;
				rate = calculate_htrate(buf[0], buf[1], buf[2]);
				//printf("ht rate=%d,buf[]=0x%02x%02x%02x\n", rate, buf[0], buf[1], buf[2]);
			} else if (type == 1) {
				ret = read_size(client_socket, buf, 8);
				if (ret < 1)
					break;
				//uint16_t known = le_to_host16((*(uint16_t *)buf));
				uint16_t known = *(uint16_t *)buf;
				rate = calculate_vhtrate(known, buf[2], buf[3] & IEEE80211_RADIOTAP_VHT_BW_MASK, &buf[4]);
			} else {
				fprintf(stderr, "unknown type(%d)\n", type);
				break;
			}

			ret = write(client_socket, &rate, sizeof(rate));
			//printf("write ret=%d\n", ret);
			if (ret < 0) {
				fprintf(stderr, "write() failed(%d)\n", ret);
				break;
			}
			//printf("ret=%d\n", ret);
			fsync(client_socket);
		}

		close(client_socket);
		//printf("client closed\n");
	}

 out:
	close(server_socket);
	unlink(PACKET_PARSER_SOCKET_PATH);
	return 0;
}
